class CollectionNames:
    @staticmethod
    def site(lang='en'):
        return f'{lang}_site'

    @staticmethod
    def links(lang='en'):
        return f'{lang}_link'

    @staticmethod
    def wikidata():
        return 'wikidata'
